==> mdcached-1.0.2.tgz: mdcached-1.0.2/mdcached/mc_client.h <==
/*-
 * This file is a part of the MDCacheD server
 * (c) 2007.-2011. Ivan Voras <ivoras@gmail.com>
 * Released under the 2-clause BSDL.
 * $Id: mc_client.h 379 2011-11-10 00:09:22Z ivoras $
 */

/**
 * Client library interface.
 * Return values note: most functions return signed integer status indicators.
 * Return values less than zero are picked from MC_RESULT... values, and results
 * larger than zero are standard operating system "errno" values. Zero results
 * indicate successful operation with nothing much to report. Additionally,
 * some functions have error_msg string arguments (char**), which may be used
 * (but don't have to, even if present) to report additional human-readable
 * information.
 *
 * This API is *not* thread-safe. Multiple threads must not use the same
 * struct mc_connection object.
 */


/*
 * Depends on:
 *	mc_protocol.h
 */

#ifdef __cplusplus
extern "C" {
#endif

#define MC_CLIENT_VERSION_ID	"$Id: mc_client.h 379 2011-11-10 00:09:22Z ivoras $"

extern unsigned short int mc_client_seq;
#define MC_SEQ (&mc_client_seq)

/**
 * MDCacheD connection structure.
 */
struct mc_connection {
	int sock;
	unsigned int ops_count;
	enum { MC_SYNC, MC_ASYNC } type;
	enum { MC_ASYNC_IDLE, MC_ASYNC_WANT_REPLY } async_state;
	void *async_buf;
	unsigned int async_buf_pos;
	unsigned int async_buf_size;
};

struct mc_multidata_result {
	uint32_t n_records;
	struct mc_resp_multidata *pkt;
	struct mc_data_entry **records;
};

struct mc_multistring_1string {
	uint16_t str_len;
	char data[];
};

struct mc_multistring_result {
	uint32_t n_records;
	struct mc_resp_multistring *pkt;
	struct mc_multistring_1string **records;
};


/* The following constants are returned as the results of all mc_* functions which return an int. */

/** OK - nothing to report */
#define MC_RESULT_OK		0
/** Server indicated something's wrong with the request, response or server state */
#define MC_RESULT_STATUS	-10
/** Network error (e.g. connection closed, etc.) */
#define MC_RESULT_NETWORK	-11
/** Searched key not found */
#define MC_RESULT_NOT_FOUND	-12
/** Server returned unexpected result */
#define MC_RESULT_UNEXPECTED	-13
/** Invalid parameter passed to client API */
#define MC_RESULT_PARAM		-14
#define MC_INVALID_PARAM	MC_RESULT_PARAM
/** Connection is in invalid state for the operation */
#define MC_RESULT_STATE		-15
/** There's more data pending (meaning depends on the operation) */
#define MC_RESULT_PENDING	-16
/** ENOMEM error */
#define MC_ENOMEM		-100

/**
 * Return human-readable string representation of error code.
 */
char *mc_result_to_string(int code);

/**
 * Connects to local unix socket. Allocates and returns a mc_connection
 * structure (or NULL).
 */
struct mc_connection *mc_connect_local(char *unix_socket, int do_handshake, char **error_msg);

/**
 * Connects to a remote server. Allocates and returns a mc_connection
 * structure (or NULL).
 */
struct mc_connection *mc_connect_tcp(char *host_name, int port, int do_handshake, char **error_msg);

/**
 * Make connection asynchronous. Asynchronous connections can only be used
 * with *async* functions. Note that this means some functions (like handshake)
 * must be performed synchronously.
 */
int mc_set_connection_async(struct mc_connection *mc);

/**
 * Closes and frees a connection structure.
 */
void mc_close_connection(struct mc_connection *mc);

/**
 * Perform a client-server handshake to verify if version numbers
 * and endianess match.
 */
int mc_perform_handshake(struct mc_connection *cn, char **error_msg);

/**
 * The simplest way to put data into the cache.
 */
int mc_put_simple(struct mc_connection *cn, char *key, unsigned int key_len, char *value, unsigned int value_len, time_t exptime, char **error_msg);

/**
 * Async version of mc_put_simple. Status report needs to be collected
 * asynchronously.
 */
int mc_put_simple_async(struct mc_connection *cn, char *key, unsigned int key_len, char *value, unsigned int value_len, time_t exptime, char **error_msg);

/**
 * The next to simplest way to put data into the cache, this time with tags.
 */
int mc_put_simple_tags(struct mc_connection *cn, char *key, unsigned int key_len, char *value, unsigned int value_len, struct mc_tag *tags, unsigned int n_tags, time_t exptime, char **error_msg);
#define mc_put mc_put_simple_tags

/**
 * The simplest way to retrieve a record from the cache.
 */
int mc_get_simple(struct mc_connection *cn, char *key, unsigned int key_len, char **data, unsigned int *data_len, char **error_msg);

/**
 * Delete a record from the cache.
 */
int mc_del_simple(struct mc_connection *cn, char *key, unsigned int key_len, char **error_msg);

/**
 * Async version of mc_get_simple. This function initiates data retrieval, then
 * returns before data is actually received from the server. Status report and
 * returned data need to be collected asynchronously.
 */
int mc_get_simple_async(struct mc_connection *cn, char *key, unsigned int key_len, char **error_msg);

/**
 * Gets a set of records which have the given tag key and any of the given tag values.
 * The result is returned as the struct mc_multidata_result, which is allocated
 * by the function and must be freed by mc_multidata_result_free().
 */
int mc_get_by_tag_values(struct mc_connection *cn, tag_key_t key, tag_value_t *tag_values, unsigned n_values, struct mc_multidata_result **result, char **error_msg);

/**
 * Frees the mc_multidata_result which was allocated & returned by one of the
 * multidata operations. Due to the internal structure of mc_multidata_result,
 * it should not be modified (or freed) by the client.
 */
void mc_multidata_result_free(struct mc_multidata_result *mr);

/**
 * Frees the mc_multistring_result which was allocated & returned by the
 * functions. Due to the internal structure of mc_multistring_result,
 * it should not be modified (or freed) by the client.
 */
void mc_multistring_result_free(struct mc_multistring_result *msr);

/**
 * Deletes a set of records which have the given tag key and any of the given tag values.
 */
int mc_del_by_tag_values(struct mc_connection *cn, tag_key_t key, tag_value_t *tag_values, unsigned n_values, unsigned *n_records, char **error_msg);

/**
 * Gets a list of all keys (record names) stored in the cache. This is not a very
 * efficient operation.
 */
int mc_get_all_keys(struct mc_connection *cn, unsigned flags, struct mc_multistring_result **mr, char **error_msg);

/**
 * Performs a simple atomic op.
 */
int mc_atomic_op(struct mc_connection *cn, int op, char *key, unsigned int key_len, int64_t arg1, int64_t arg2, int64_t *result, char **error_msg);

/* Shortcuts for simple atomic ops */
#define mc_atomic_cmpset(cn, key, key_len, arg1, arg2, mr, err) mc_atomic_op(cn, MCMD_ATOMIC_CMPSET, key, key_len, arg1, arg2, mr, err)
#define mc_atomic_add(cn, key, key_len, arg1, mr, err) mc_atomic_op(cn, MCMD_ATOMIC_ADD, key, key_len, arg1, 0, mr, err)
#define mc_atomic_fetchadd(cn, key, key_len, arg1, mr, err) mc_atomic_op(cn, MCMD_ATOMIC_FETCHADD, key, key_len, arg1, 0, mr, err)
#define mc_atomic_readandclear(cn, key, key_len, mr, err) mc_atomic_op(cn, MCMD_ATOMIC_READANDCLEAR, key, key_len, 0, 0, mr, err)

/**
 * Puts multiple records into the cache.
 */
int mc_mput(struct mc_connection *cn, uint16_t n_records, struct mc_data_entry **records, unsigned flags, char **error_msg);

/**
 * Gets a set of named records.
 */
int mc_mget(struct mc_connection *cn, uint16_t n_records, uint16_t *key_lengths, char **keys, unsigned flags, struct mc_multidata_result **result, char **error_msg);

/**
 * Deletes a set of named records.
 */
int mc_mdel(struct mc_connection *cn, uint16_t n_records, uint16_t *key_lengths, char **keys, unsigned flags, unsigned *n_deleted, char **error_msg);

/**
 * Puts a record to the server containing the specified data and tags but without
 * a key. The key is auto-generated on the server and returned back.
 */
int mc_tstack_push(struct mc_connection *cn, struct mc_tag *tags, unsigned n_tags, char *data, unsigned data_size, char **new_key, unsigned *new_key_size, char **error_msg);

/**
 * "Pops" a record off a virtual stack on the server identified by the tag (i.e.
 * its key and value).
 */ 
int mc_tstack_pop(struct mc_connection *cn, tag_key_t tkey, tag_value_t tval, struct mc_data_entry **mce, char **error_msg);


/**
 * Do a "send" cycle of async communication - is only usable to call when
 * mc_put_simple_async or similar functions return MC_RESULT_PENDING. The
 * function will return MC_RESULT_OK if all data has been sent, or
 * MC_RESULT_PENDING if there's more data pending in the buffers.
 */
int mc_async_sendpoll(struct mc_connection *cn);

/**
 * Do a "receive" cycle of async communication - is only usable when expecting
 * an answere from async communication, and then if some other means of polling
 * (poll(), kqueue) indicate there's something to receive. If a whole response
 * has been received, the function will return MC_RESULT_OK and outpkt will be
 * set (free it manually when done using it), otherwise, MC_RESULT_PENDING will
 * be returned.
 */
int mc_async_recvpoll(struct mc_connection *cn, struct mc_header **outpkt);

#ifdef __cplusplus
}
#endif

==> mdcached-1.0.3.tgz: mdcached-1.0.3/mdcached/mc_client.h <==
/*-
 * This file is a part of the MDCacheD server
 * (c) 2007.-2011. Ivan Voras <ivoras@gmail.com>
 * Released under the 2-clause BSDL.
 * $Id: mc_client.h 436 2012-03-05 20:01:55Z ivoras $
 */

/**
 * Client library interface.
 * Return values note: most functions return signed integer status indicators.
 * Return values less than zero are picked from MC_RESULT... values, and results
 * larger than zero are standard operating system "errno" values. Zero results
 * indicate successful operation with nothing much to report. Additionally,
 * some functions have error_msg string arguments (char**), which may be used
 * (but don't have to, even if present) to report additional human-readable
 * information.
 *
 * This API is *not* thread-safe. Multiple threads must not use the same
 * struct mc_connection object.
 */


/*
 * Depends on:
 *	mc_protocol.h
 */

#ifdef __cplusplus
extern "C" {
#endif

#define MC_CLIENT_VERSION_ID	"$Id: mc_client.h 436 2012-03-05 20:01:55Z ivoras $"

extern unsigned short int mc_client_seq;
#define MC_SEQ (&mc_client_seq)

/**
 * MDCacheD connection structure.
 */
struct mc_connection {
	int sock;
	unsigned int ops_count;
	enum { MC_SYNC, MC_ASYNC } type;
	enum { MC_ASYNC_IDLE, MC_ASYNC_WANT_REPLY } async_state;
	void *async_buf;
	unsigned int async_buf_pos;
	unsigned int async_buf_size;
};

struct mc_multidata_result {
	uint32_t n_records;
	struct mc_resp_multidata *pkt;
	struct mc_data_entry **records;
};

struct mc_multistring_1string {
	uint16_t str_len;
	char data[];
};

struct mc_multistring_result {
	uint32_t n_records;
	struct mc_resp_multistring *pkt;
	struct mc_multistring_1string **records;
};


/* The following constants are returned as the results of all mc_* functions which return an int. */

/** OK - nothing to report */
#define MC_RESULT_OK		0
/** Server indicated something's wrong with the request, response or server state */
#define MC_RESULT_STATUS	-10
/** Network error (e.g. connection closed, etc.) */
#define MC_RESULT_NETWORK	-11
/** Searched key not found */
#define MC_RESULT_NOT_FOUND	-12
/** Server returned unexpected result */
#define MC_RESULT_UNEXPECTED	-13
/** Invalid parameter passed to client API */
#define MC_RESULT_PARAM		-14
#define MC_INVALID_PARAM	MC_RESULT_PARAM
/** Connection is in invalid state for the operation */
#define MC_RESULT_STATE		-15
/** There's more data pending (meaning depends on the operation) */
#define MC_RESULT_PENDING	-16
/** ENOMEM error */
#define MC_ENOMEM		-100

/**
 * Return human-readable string representation of error code.
 */
char *mc_result_to_string(int code);

/**
 * Connects to local unix socket. Allocates and returns a mc_connection
 * structure (or NULL).
 */
struct mc_connection *mc_connect_local(char *unix_socket, int do_handshake, char **error_msg);

/**
 * Connects to a remote server. Allocates and returns a mc_connection
 * structure (or NULL).
 */
struct mc_connection *mc_connect_tcp(char *host_name, int port, int do_handshake, char **error_msg);

/**
 * Make connection asynchronous. Asynchronous connections can only be used
 * with *async* functions. Note that this means some functions (like handshake)
 * must be performed synchronously.
 */
int mc_set_connection_async(struct mc_connection *mc);

/**
 * Closes and frees a connection structure.
 */
void mc_close_connection(struct mc_connection *mc);

/**
 * Perform a client-server handshake to verify if version numbers
 * and endianess match.
 */
int mc_perform_handshake(struct mc_connection *cn, char **error_msg);

/**
 * The simplest way to put data into the cache.
 */
int mc_put_simple(struct mc_connection *cn, char *key, unsigned int key_len, char *value, unsigned int value_len, time_t exptime, char **error_msg);

/**
 * Async version of mc_put_simple. Status report needs to be collected
 * asynchronously.
 */
int mc_put_simple_async(struct mc_connection *cn, char *key, unsigned int key_len, char *value, unsigned int value_len, time_t exptime, char **error_msg);

/**
 * The next to simplest way to put data into the cache, this time with tags.
 */
int mc_put_simple_tags(struct mc_connection *cn, char *key, unsigned int key_len, char *value, unsigned int value_len, struct mc_tag *tags, unsigned int n_tags, time_t exptime, char **error_msg);
#define mc_put mc_put_simple_tags

/**
 * The simplest way to retrieve a record from the cache.
 */
int mc_get_simple(struct mc_connection *cn, char *key, unsigned int key_len, char **data, unsigned int *data_len, char **error_msg);

/**
 * Delete a record from the cache.
 */
int mc_del_simple(struct mc_connection *cn, char *key, unsigned int key_len, char **error_msg);

/**
 * Async version of mc_get_simple. This function initiates data retrieval, then
 * returns before data is actually received from the server. Status report and
 * returned data need to be collected asynchronously.
 */
int mc_get_simple_async(struct mc_connection *cn, char *key, unsigned int key_len, char **error_msg);

/**
 * Gets a set of records which have the given tag key and any of the given tag values.
 * The result is returned as the struct mc_multidata_result, which is allocated
 * by the function and must be freed by mc_multidata_result_free().
 */
int mc_get_by_tag_values(struct mc_connection *cn, tag_key_t key, tag_value_t *tag_values, unsigned n_values, struct mc_multidata_result **result, char **error_msg);

/**
 * Frees the mc_multidata_result which was allocated & returned by one of the
 * multidata operations. Due to the internal structure of mc_multidata_result,
 * it should not be modified (or freed) by the client.
 */
void mc_multidata_result_free(struct mc_multidata_result *mr);

/**
 * Frees the mc_multistring_result which was allocated & returned by the
 * functions. Due to the internal structure of mc_multistring_result,
 * it should not be modified (or freed) by the client.
 */
void mc_multistring_result_free(struct mc_multistring_result *msr);

/**
 * Deletes a set of records which have the given tag key and any of the given tag values.
 */
int mc_del_by_tag_values(struct mc_connection *cn, tag_key_t key, tag_value_t *tag_values, unsigned n_values, unsigned *n_records, char **error_msg);

/**
 * Gets a list of all keys (record names) stored in the cache. This is not a very
 * efficient operation.
 */
int mc_get_all_keys(struct mc_connection *cn, unsigned flags, struct mc_multistring_result **mr, char **error_msg);

/**
 * Performs a simple atomic op.
 */
int mc_atomic_op(struct mc_connection *cn, int op, char *key, unsigned key_len, int64_t arg1, int64_t arg2, int64_t *result, char **error_msg);

/* Shortcuts for simple atomic ops */

/**
 * ATOMIC_CMPSET: Compares if the stored value is equal to arg1, and if it is, 
 * sets it to arg2. Returns 1 in *res if the store has suceeded or 0 if it hasn't.
 */
int mc_atomic_cmpset(struct mc_connection *cn, char *key, unsigned key_len, int64_t arg1, int64_t arg2, int64_t *res, char **error_msg);

/**
 * ATOMIC_ADD: Adds the value in arg1 to the stored value, returns the new sum in *res.
 */
int mc_atomic_add(struct mc_connection *cn, char *key, unsigned key_len, int64_t arg1, int64_t *res, char **error_msg);

/**
 * ATOMIC_FETCHADD: Adds the value in arg1 to the stored value, returns the OLD value in *res.
 */
int mc_atomic_fetchadd(struct mc_connection *cn, char *key, unsigned key_len, int64_t arg1, int64_t *res, char **error_msg);

/**
 * ATOMIC_READANDCLEAR: Clears the value, but returns the old value before clearing in *res.
 */
int mc_atomic_readandclear(struct mc_connection *cn, char *key, unsigned key_len, int64_t *res, char **error_msg);

/**
 * Initializes a record for use with atomic ops with the given value and tags.
 * The other atomic ops do not modify the tags of the record.
 */
int mc_atomic_put(struct mc_connection *cn, char *key, unsigned key_len, int64_t value, struct mc_tag *tags, unsigned n_tags, time_t exptime, char **error_msg);

/**
 * Retrieves the stored value of the record used with atomic ops. Similar in effect
 * to FETCHADD(0) but allows better concurrency.
 */
int mc_atomic_get(struct mc_connection *cn, char *key, unsigned key_len, int64_t *res, char **error_msg);

/**
 * Puts multiple records into the cache.
 */
int mc_mput(struct mc_connection *cn, uint16_t n_records, struct mc_data_entry **records, unsigned flags, char **error_msg);

/**
 * Gets a set of named records.
 */
int mc_mget(struct mc_connection *cn, uint16_t n_records, uint16_t *key_lengths, char **keys, unsigned flags, struct mc_multidata_result **result, char **error_msg);

/**
 * Deletes a set of named records.
 */
int mc_mdel(struct mc_connection *cn, uint16_t n_records, uint16_t *key_lengths, char **keys, unsigned flags, unsigned *n_deleted, char **error_msg);

/**
 * Puts a record to the server containing the specified data and tags but without
 * a key. The key is auto-generated on the server and returned back.
 */
int mc_tstack_push(struct mc_connection *cn, struct mc_tag *tags, unsigned n_tags, char *data, unsigned data_size, char **new_key, unsigned *new_key_size, char **error_msg);

/**
 * "Pops" a record off a virtual stack on the server identified by the tag (i.e.
 * its key and value).
 */ 
int mc_tstack_pop(struct mc_connection *cn, tag_key_t tkey, tag_value_t tval, struct mc_data_entry **mce, char **error_msg);


/**
 * Do a "send" cycle of async communication - is only usable to call when
 * mc_put_simple_async or similar functions return MC_RESULT_PENDING. The
 * function will return MC_RESULT_OK if all data has been sent, or
 * MC_RESULT_PENDING if there's more data pending in the buffers.
 */
int mc_async_sendpoll(struct mc_connection *cn);

/**
 * Do a "receive" cycle of async communication - is only usable when expecting
 * an answere from async communication, and then if some other means of polling
 * (poll(), kqueue) indicate there's something to receive. If a whole response
 * has been received, the function will return MC_RESULT_OK and outpkt will be
 * set (free it manually when done using it), otherwise, MC_RESULT_PENDING will
 * be returned.
 */
int mc_async_recvpoll(struct mc_connection *cn, struct mc_header **outpkt);

#ifdef __cplusplus
}
#endif
